import sys
import json
import argparse
from datetime import datetime
from typing import Any

import requests


class LogEntry:
    """Class describing a connection attempt"""

    def __init__(self: 'LogEntry', log_entry: str) -> None:
        self.date = datetime.strptime(log_entry[1:20], '%Y-%m-%d %H:%M:%S')
        self.host = log_entry.split('[')[2].split(']')[0]
        self.failed = 'FAIL' in log_entry


def printOutages(config: dict) -> None:
    """Print outages from a log file"""

    with open(config['log'], 'r') as file:
        log = list(map(lambda x: LogEntry(x), file.readlines()))

    # cluster log entries by host
    log_by_host = {}
    for entry in log:
        if entry.host not in log_by_host:
            log_by_host[entry.host] = [entry]
        else:
            log_by_host[entry.host].append(entry)

    # cluster fails to outages
    # we consider the time of subsequent failed connection attemts to the same host an outage
    outages: list = []
    last_failed = False
    for entries in log_by_host.values():
        for entry in entries:
            if entry.failed and last_failed:
                outages[-1].append(entry)
            if entry.failed and not last_failed:
                outages.append([entry])
                last_failed = True
            if not entry.failed and last_failed:
                last_failed = False

    # print the outages by date of first fail
    for outage in sorted(outages, key=lambda x: x[0].date):

        # get duration of outage in hours and minutes
        outage_duration = round(
            (outage[-1].date - outage[0].date).seconds / 60)
        # continue if outage is shorter than the minimum duration
        if outage_duration < config['outages']:
            continue
        hours = outage_duration // 60
        minutes = outage_duration - (hours * 60)
        host = outage[0].host

        # Outputs outages in the form of "Outage at: 2023-19-01 06:29:01 lasting for 2 Hours and 39 Minutes"
        print(f"[{outage[0].date}][{host}] lasting for {'{} Hours and '.format(hours) if hours >= 1 else ''}{minutes} Minutes")


def isOnline(host: str, timeout: int) -> bool:
    """Check if connection to a host can be established"""

    try:
        requests.head(f"https://{host}", timeout=timeout)
        return True
    except requests.ConnectionError:
        return False


def log(path: str, host: str, timeout: int, stdout: bool) -> None:
    """Log the connection status of a host"""

    logline = f"[{datetime.now().strftime('%Y-%m-%d %H:%M:%S')}][{host}][{'OK' if isOnline(host, timeout) else 'FAIL'}]\n"

    # TODO don't print to log path when no path is given
    if config['log'] != None:
        with open(path, 'a') as file:
            file.write(logline)

    if stdout:
        print(logline)


def cliLog(args: Any) -> None:
    # TODO read config file
    log(args.path, args.host, args.timeout, args.stdout)


def cliPOutages(args: Any) -> None:
    printOutages(args.path, args.min_duration)


def main():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()

    parser_log = subparsers.add_parser('log')
    parser_log.add_argument('--config', type=argparse.FileType('w', encoding='UTF-8'), default='./config.json',
                            help='Path to a config file in JSON format. To set a command line argument, use it as a key and set its value. Config is not used unless explicitly set.')
    parser_log.add_argument('--path', type=str, default='./connection.log',
                            help='Path to a the file where results should be appended to. Created the file if it does not already exist.')
    parser_log.add_argument('--host', type=str, default='1.1.1.1',
                            help='IP or hostname of a server to query.')
    parser_log.add_argument('--timeout', type=int,
                            default=2, help='Timeout in seconds.')
    parser_log.add_argument('--stdout', default=False,
                            action='store_true', help='Print the logline to stdout.')
    parser_log.set_defaults(func=printOutages)

    parser_outages = subparsers.add_parser('outages')
    parser_outages.add_argument('--min-duration', type=int, default=3,
                                help='Set the minimum duration in minutes where a connection loss is condidered an outage.')
    parser_outages.add_argument(
        '--path', type=str, default='./connection.log', help='Path to the file with outage data.')
    parser_outages.set_defaults(func=printOutages)

    args = parser.parse_args()
    # args.func(args)

    print(args)


if __name__ == '__main__':
    main()
